from __future__ import absolute_import, unicode_literals

from datetime import date, datetime

from operator import attrgetter

from django import template
from django.utils.timezone import now

register = template.Library()


@register.filter
def next_start_dt(event, after_date=None):
    """
    Get the first occurrence of an event after a given date.
    If no occurrence is found, the first occurrence is returned.
    """
    if not isinstance(after_date, (date, datetime)):
        after_date = now()
    dt = event.dateandtimes.filter(day__gte=after_date).first()
    if dt is None:
        return event.dateandtimes.first()
    return dt


@register.filter
def annotate_start_dts(events, after_date=None):
    """
    Annotate each Event in a queryset with the next upcoming occurrence (EventDateTime).
    Then sort by this new computed field.
    """
    for event in list(events):
        event.next_start_dt = next_start_dt(event, after_date)
    return sorted(events, key=attrgetter("next_start_dt.day"))



from __future__ import unicode_literals

import bleach

from django import template
from django.core.exceptions import ImproperlyConfigured

from mezzanine.conf import settings

register = template.Library()


@register.filter
def microtemplate(value):
    """
    Filter that parses certain variables in the value param.
    It can be used to allow some templating capabilities in admin fields.
    Default template tokens can be found in mezzy.defaults.
    """
    import re
    from mezzanine.utils.importing import import_dotted_path

    tokens = import_dotted_path(settings.MEZZY_MICROTEMPLATE_TOKENS)
    for pattern, repl in tokens:
        value = re.sub(pattern, repl, value)
    return value


@register.simple_tag
def load_theme():
    """
    Adds the `SiteConfiguration` to the context.
    It also uses the request object as cache to avoid some DB hits.
    """
    from mezzanine.core.request import current_request

    try:
        from theme.models import SiteConfiguration
    except ImportError:
        raise ImproperlyConfigured(
            "You must create a SiteConfiguration model inside the theme app to use the "
            "load_theme tag."
        )

    request = current_request()
    if hasattr(request, "theme"):
        return request.theme
    theme = SiteConfiguration.objects.get_or_create()[0]
    request.theme = theme
    return theme


@register.filter
def clean(value):
    """
    Removes HTML tags and attributes that aren't allowed
    for Mezzanine by default, even if the user has disabled filtering.
    """
    kwargs = {
        "tags": settings.RICHTEXT_ALLOWED_TAGS,
        "attrs": settings.RICHTEXT_ALLOWED_ATTRIBUTES,
        "styles": settings.RICHTEXT_ALLOWED_STYLES,
        "strip": True,
        "strip_comments": True,
    }
    return bleach.clean(value, **kwargs)



from __future__ import absolute_import, unicode_literals

from mezzanine.core.forms import Html5Mixin


class UXFormMixin(Html5Mixin):
    """
    Form tweaks for a better user experience.
    """

    def __init__(self, *args, **kwargs):
        super(UXFormMixin, self).__init__(*args, **kwargs)

        for field in self.fields.values():
            # Annotate each field with their own field type
            # so templates can add field.type as a class for styling
            field.type = field.widget.__class__.__name__.lower()
            # Remove the autofocus added by Html5Mixin
            if field.widget.attrs.get("autofocus") == "":
                del field.widget.attrs["autofocus"]



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from mezzanine.core.models import Slugged, Orderable


@python_2_unicode_compatible
class Titled(models.Model):
    """
    Inline model with a title.
    """

    title = models.CharField("Title", max_length=100)

    class Meta:
        abstract = True

    def __str__(self):
        return self.title


class TitledInline(Orderable, Titled):
    """
    Inline model with a title and manual sorting.
    """

    class Meta:
        abstract = True


class SluggedInline(Orderable, Slugged):
    """
    Inline model with a title, a slug and manual sorting.
    """

    class Meta:
        abstract = True



from __future__ import absolute_import, unicode_literals

from datetime import timedelta

from django.contrib.auth.models import AnonymousUser
from django.contrib.sessions.middleware import SessionMiddleware
from django.shortcuts import Http404
from django.test import RequestFactory
from django.utils.timezone import now

from mezzanine.conf import settings

# Helpers to quickly set dates to the past or future
yesterday = now() + timedelta(days=-1)
tomorrow = now() + timedelta(days=1)


class ViewTestMixin(object):
    """
    Helper methods to test views directly, without URL resolution.
    """

    def __init__(self, *args, **kwargs):
        self.middleware = SessionMiddleware()
        super(ViewTestMixin, self).__init__(*args, **kwargs)

    def execute(self, method, cls_or_func, *args, **kwargs):
        """
        Call a view (class or function-based) with a specific HTTP method.
        All args and kwargs will be passed to the view, with the exception of:

        - `data` will be used to build the request's GET or POST data
        - `user` will be attached to the request (will fall back to AnonymousUser)

        The resulting request will always have a user object and session data.
        """
        data = kwargs.pop("data", {})
        request = getattr(RequestFactory(), method)("/custom-request/", data)
        request.user = kwargs.pop("user", AnonymousUser())

        self.middleware.process_request(request)
        request.session.save()

        if hasattr(cls_or_func, "as_view"):
            return cls_or_func.as_view()(request, *args, **kwargs)
        return cls_or_func(request, *args, **kwargs)

    def get(self, cls_or_func, *args, **kwargs):
        return self.execute("get", cls_or_func, *args, **kwargs)

    def post(self, cls_or_func, *args, **kwargs):
        return self.execute("post", cls_or_func, *args, **kwargs)

    def assert200(self, cls_or_func, *args, **kwargs):
        response = self.get(cls_or_func, *args, **kwargs)
        self.assertEqual(response.status_code, 200)
        return response

    def assert404(self, cls_or_func, *args, **kwargs):
        with self.assertRaises(Http404):
            return self.get(cls_or_func, *args, **kwargs)

    def assertLoginRequired(self, cls_or_func, *args, **kwargs):
        response = self.get(cls_or_func, *args, **kwargs)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(
            response["location"].startswith("%s?next=" % settings.LOGIN_URL)
        )
        return response
